@@ -3,7 +3,7 @@
 #
 #  signed/vector valued measures with atomic and diffuse components
 #
-#  $Revision: 1.105 $  $Date: 2022/06/19 04:08:19 $
+#  $Revision: 1.110 $  $Date: 2026/01/19 05:50:21 $
 #
 msr <- function(qscheme, discrete, density, check=TRUE) {
   if(!is.quad(qscheme))
@@ -43,9 +43,9 @@ msr <- function(qscheme, discrete, density, check=TRUE) {
     if(check) {
       # check numbers of rows
       check.nmatrix(discrete, ndata, things="data points",
-                    naok=TRUE, squarematrix=FALSE)
+                    naok=TRUE, squarematrix=FALSE, mname="discrete")
       check.nmatrix(density,  nquad, things="quadrature points",
-                    naok=TRUE, squarematrix=FALSE)
+                    naok=TRUE, squarematrix=FALSE, mname="density")
     }
     nd <- ncol(discrete)
     nc <- ncol(density)
@@ -81,15 +81,19 @@ msr <- function(qscheme, discrete, density, check=TRUE) {
     discretepad <- ok * discretepad
     density     <- ok * density
   }
-  
+
   ## finished
-  out <- list(loc = U,
-              val = val,
-              atoms = Z,
-              discrete = discretepad,
-              density = density,
-              wt = wt)
+  out <- list(loc       = U,
+              val       = val,
+              atoms     = Z,
+              discrete  = discretepad,
+              density   = density,
+              wt        = wt)
   class(out) <- "msr"
+
+  ## possibly add information about the original context (eg linear network)
+  attr(out, "situ") <- attr(qscheme, "situ")
+  
   return(out)
 }
 
@@ -204,6 +208,15 @@ split.msr <- function(x, f, drop=FALSE, ...) {
                    SIMPLIFY=FALSE)
   names(result) <- names(locsplit)
   result <- lapply(result, "class<-", value="msr")
+  ## also split the pattern of auxiliary locations if present
+  if(!is.null(z <- attr(x, "situ"))) {
+    zsplit <- split(z, g, drop=drop)
+    result <- mapply("attr<-",
+                     x=result,
+                     which="situ",
+                     value=zsplit,
+                     SIMPLIFY=FALSE)
+  }
   if(drop && any(isnul <- (sapply(locsplit, npoints) == 0)))
     result[isnul] <- NULL
   result <- as.solist(result)
@@ -444,6 +457,8 @@ plot.msr <- function(x, ..., add=FALSE,
               density=dens,
               wt=wt)
   class(out) <- "msr"
+  if(!is.null(z <- attr(x, "situ")))
+    attr(out, "situ") <- if(missing(i)) z else z[id]
   return(out)    
 }
 
@@ -701,7 +716,7 @@ harmonise.msr <- local({
     if(!any(ismeasure))
       stop("No measures supplied")
     if(!all(ismeasure))
-    stop("All arguments should be measures (objects of class msr)")
+      stop("All arguments should be measures (objects of class msr)")
     if(n < 2) return(argz)
     result <- vector(mode="list", length=n)
     ## extract entries
@@ -841,8 +856,22 @@ residualMeasure <- function(Q, lambda,
   P <- union.quad(Q)
   Lambda <- if(is.im(lambda)) {
               lambda[P, drop=FALSE]
-            } else {
+            } else if(is.list(lambda) && all(sapply(lambda, is.im))) {
               sapply(lambda, "[", i=unmark(P), drop=FALSE)
+            } else if(is.numeric(lambda) && length(lambda) == 1) {
+              rep(lambda, npoints(P))
+            } else if(is.function(lambda)) {
+              lamP <- lambda(P$x, P$y)
+              if((nl <- length(lamP)) != (nP <- npoints(P)))
+                stop(paste("Function lambda incorrectly returned", nl,
+                           ngettext(nl, "value", "values"),
+                           "when applied to", nP, "data points"),
+                     call.=FALSE)
+              lamP
+            } else {
+              stop(paste("Argument lambda should be an image,",
+                         "a list of images, a function, or a single number"),
+                   call.=FALSE)
             }
   switch(type,
          raw = {
